/*
TASK:F
LANG:C++
*/

#include <cstdio>
#include <cstdlib>
#include <utility>
#include <algorithm>

const int MAXN = 1 << 10;
const int MAXM = 1 << 12;
const double _infinity = 1e9;

struct el {
	int to;
	double dist;
	int next;
	el () {}
	el (int _to, double _dist, int _next) : to (_to), dist (_dist), next (_next) {}
};

struct arc {
	int a, b;
	double dist;
	arc () {}
	arc (int _a, int _b, double _dist) : a (_a), b (_b), dist (_dist) {}
};

struct heap {
	int to;
	double dist;
	heap () {}
	heap (int _to, double _dist) : to (_to), dist (_dist) {}
};

int N, M;
int vec[MAXN];
el buf[MAXM << 1];
arc arcs[MAXM];

double ma3x[MAXN][MAXN];

inline double min (double a, double b) {return a < b ? a : b;}

///heap
const int MAXHLEN = MAXN << 1;
const int POPPED = -2, NOT_IN = -1;

heap h[MAXHLEN];
int idx[MAXN];
int hlen;
/*
void prnt () {
	int ub = 2;
	int i = 1;
	while (ub < 16) {
		for (; i < ub; ++i)
			printf ("(%d %.2lf)", h[i].to, h[i].dist);
		printf ("\n");
		ub *= 2;
	}
}
*/
inline void init () {
	static int i;
	for (i = 1; i < MAXHLEN; ++i)
		h[i].dist = _infinity;
	h[0].dist = -_infinity;
	hlen = 1;
	for (i = 0; i < MAXN; ++i)
		idx[i] = NOT_IN;
}

inline heap pop () {
	heap ret = h[1];
	idx[ret.to] = POPPED;
	heap tomove = h[--hlen];
	h[hlen].dist = _infinity;
	if (hlen == 1) return ret;

	int p1 = 1, p2 = 2;
	for (;;) {
		if (h[p2+1].dist < h[p2].dist)
			++p2;
		if (h[p2].dist < tomove.dist) {
			h[p1] = h[p2];
			idx[h[p2].to] = p1;
		
			p1 = p2;
			p2 *= 2;
		} else
			break;
	}
	h[p1] = tomove;
	idx[tomove.to] = p1;

	return ret;
}

inline void siftup (int id) {
	heap tomove = h[idx[id]];
	int p1 = idx[id], p2 = p1 / 2;
	
	while (h[p2].dist > tomove.dist) {
		h[p1] = h[p2];
		idx[h[p2].to] = p1;
		
		p1 = p2;
		p2 /= 2;
	}

	h[p1] = tomove;
	idx[id] = p1;
}

inline void add (int to, double dist) {
	if (idx[to] == NOT_IN) {
//		printf ("add (%d, %lf)\n", to, dist);
		h[hlen] = heap (to, dist);
		idx[to] = hlen++;
		siftup (to);
//		prnt ();
	} else if (idx[to] != POPPED && h[idx[to]].dist > dist) {
//		printf ("upd (%d, %lf) - %lf\n", to, h[idx[to]].dist, dist);
		h[idx[to]].dist = dist;
		siftup (to);
//		prnt ();
	}
}
///end heap

void dijkstra (int s) {
	static heap crnt;
	static int i;
	init ();
	add (s, 0);
	
	while (hlen > 1) {
		crnt = pop ();
//		printf ("popped %d %lf\n", crnt.to, crnt.dist);
//		prnt ();
		ma3x[s][crnt.to] = crnt.dist;
		for (i = vec[crnt.to]; i != -1; i = buf[i].next)
			add (buf[i].to, buf[i].dist + crnt.dist);
	}
}

std::pair <double, double> be[MAXN];
double res = _infinity;//minimize it

inline bool cmp (const std::pair <double, double> &a, const std::pair <double, double> &b) {
	return a.first > b.first;
}

double _d;
inline double calc (double e1, double b2) {
	return (_d + e1 + b2) / 2.;
}

int _f, _s;
void f (double d) {
	static int i;
	_d = d;
	std::sort (be, be + N, cmp);
/*	printf ("-----\n");
	for (i = 0; i < N; ++i)
		printf ("%lf %lf\n", be[i].first, be[i].second);
*/
	double cend = be[0].second;
	res = min (res, be[0].first);
	for (i = 1; i < N; ++i)
		if (be[i].second > cend) {
//			printf ("%d -- %lf(%lf %lf)\n", i, calc (cend, be[i].first), cend, be[i].first);
			res = min (res, calc (cend, be[i].first));
			cend = be[i].second;
		}
	res = min (res, cend);
//	printf ("last cend -- %lf\n", cend);
}

int main () {
	scanf ("%d %d", &N, &M);
	
	int i, j;
	int a, b;
	double d;
	int bp = 0;
	for (i = 0; i < N; ++i) vec[i] = -1;
	for (i = 0; i < M; ++i) {
		scanf ("%d %d %lf", &a, &b, &d);
//		printf ("%d ", i);
		--a; --b;
		buf[bp] = el (b, d, vec[a]);
		vec[a] = bp++;
		
		buf[bp] = el (a, d, vec[b]);
		vec[b] = bp++;
		arcs[i] = arc (a, b, d);
	}
//	printf ("\ngot input!\n");

	for (i = 0; i < N; ++i) for (j = 0; j < N; ++j) ma3x[i][j] = _infinity;

	for (i = 0; i < N; ++i)
		dijkstra (i);

//	printf ("after dijkstra!\n");
//	for (i = 0; i < N; ++i) {for (j = 0; j < N; ++j) printf ("%.2lf ", ma3x[i][j]); printf ("\n");}
	for (i = 0; i < M; ++i) {
		for (j = 0; j < N; ++j) {
			be[j] = std::make_pair (ma3x[arcs[i].a][j], ma3x[arcs[i].b][j]);
//			printf ("__%lf %lf\n", be[j].first, be[j].second);
		}
		_f = arcs[i].a;
		_s = arcs[i].b;
//		printf ("___%d %d -- %lf___\n", arcs[i].a, arcs[i].b, arcs[i].dist);
		f (arcs[i].dist);
	}

	printf ("%.4lf\n", res);

	return 0;
}
